//---     
#include "..\MultiSymbolExpert.mq5"
//---   
#include "Enums.mqh"
#include "InitializeArrays.mqh"
#include "Errors.mqh"
#include "TradeSignals.mqh"
#include "TradeFunctions.mqh"
#include "ToString.mqh"
#include "Auxiliary.mqh"
//+------------------------------------------------------------------+
//|                          |
//+------------------------------------------------------------------+
void GetSpyHandles()
  {
//---    
   for(int s=0; s<NUMBER_OF_SYMBOLS; s++)
     {
      //---     
      if(Symbols[s]!="")
        {
         //---     
         if(spy_indicator_handles[s]==INVALID_HANDLE)
           {
            //---   
            spy_indicator_handles[s]=iCustom(Symbols[s],_Period,"EventsSpy.ex5",ChartID(),0,CHARTEVENT_TICK);
            //---      
            if(spy_indicator_handles[s]==INVALID_HANDLE)
               Print("     "+Symbols[s]+"");
           }
        }
     }
  }
//+------------------------------------------------------------------+
//|                                         |
//+------------------------------------------------------------------+
void GetIndicatorHandles()
  {
//---    
   for(int s=0; s<NUMBER_OF_SYMBOLS; s++)
     {
      //---      
      if(Symbols[s]!="")
        {
         //---     
         if(signal_indicator_handles[s]==INVALID_HANDLE)
           {
            //---   
            signal_indicator_handles[s]=iMA(Symbols[s],_Period,IndicatorPeriod[s],0,MODE_SMA,PRICE_CLOSE);
            //---      
            if(signal_indicator_handles[s]==INVALID_HANDLE)
               Print("       "+Symbols[s]+"!");
           }
        }
     }
  }
//+------------------------------------------------------------------+
//|                                             |
//+------------------------------------------------------------------+
void GetBarsData(int symbol_number)
  {
//---        
   int NumberOfBars=3;
//---     (... 3 2 1 0)
   ArraySetAsSeries(open[symbol_number].value,true);
   ArraySetAsSeries(high[symbol_number].value,true);
   ArraySetAsSeries(low[symbol_number].value,true);
   ArraySetAsSeries(close[symbol_number].value,true);
//---    ,  
//       
//---    
   if(CopyClose(Symbols[symbol_number],_Period,0,NumberOfBars,close[symbol_number].value)<NumberOfBars)
     {
      Print("    ("
            +Symbols[symbol_number]+"; "+TimeframeToString(_Period)+")    Close! "
            " ("+IntegerToString(GetLastError())+"): "+ErrorDescription(GetLastError())+"");
     }
//---    
   if(CopyOpen(Symbols[symbol_number],_Period,0,NumberOfBars,open[symbol_number].value)<NumberOfBars)
     {
      Print("    ("
            +Symbols[symbol_number]+"; "+TimeframeToString(_Period)+")    Open! "
            " ("+IntegerToString(GetLastError())+"): "+ErrorDescription(GetLastError())+"");
     }
//---    
   if(CopyHigh(Symbols[symbol_number],_Period,0,NumberOfBars,high[symbol_number].value)<NumberOfBars)
     {
      Print("    ("
            +Symbols[symbol_number]+"; "+TimeframeToString(_Period)+")    High! "
            " ("+IntegerToString(GetLastError())+"): "+ErrorDescription(GetLastError())+"");
     }
//---    
   if(CopyLow(Symbols[symbol_number],_Period,0,NumberOfBars,low[symbol_number].value)<NumberOfBars)
     {
      Print("    ("
            +Symbols[symbol_number]+"; "+TimeframeToString(_Period)+")    Low! "
            " ("+IntegerToString(GetLastError())+"): "+ErrorDescription(GetLastError())+"");
     }
  }
//+------------------------------------------------------------------+
//|                                       |
//+------------------------------------------------------------------+
bool GetIndicatorsData(int symbol_number)
  {
//---           
   int NumberOfValues=3;
//---     
   if(signal_indicator_handles[symbol_number]!=INVALID_HANDLE)
     {
      //---     (... 3 2 1 0)
      ArraySetAsSeries(indicator[symbol_number].value,true);
      //---   
      if(CopyBuffer(signal_indicator_handles[symbol_number],0,0,NumberOfValues,indicator[symbol_number].value)<NumberOfValues)
        {
         Print("    ("+
               Symbols[symbol_number]+"; "+TimeframeToString(_Period)+")   indicator!  ("+
               IntegerToString(GetLastError())+"): "+ErrorDescription(GetLastError())+"");
         return(false);
        }
      return(true);
     }
//---     , ...
   else
     {
      // ...    
      GetIndicatorHandles();
     }
   return(false);
  }
//+------------------------------------------------------------------+
//|                                         |
//+------------------------------------------------------------------+
ENUM_ORDER_TYPE GetTradingSignal(int symbol_number)
  {
//---   
   if(!pos.exists)
     {
      //---    :
      if(GetSignal(symbol_number)==ORDER_TYPE_SELL)
         return(ORDER_TYPE_SELL);
      //---    :
      if(GetSignal(symbol_number)==ORDER_TYPE_BUY)
         return(ORDER_TYPE_BUY);
     }
//---   
   if(pos.exists)
     {
      //---   
      GetPositionProperties(symbol_number,P_TYPE);
      //---    
      GetPositionProperties(symbol_number,P_PRICE_LAST_DEAL);

      //---    :
      if(pos.type==POSITION_TYPE_BUY && 
         GetSignal(symbol_number)==ORDER_TYPE_SELL)
         return(ORDER_TYPE_SELL);
      //---
      if(pos.type==POSITION_TYPE_SELL && 
         GetSignal(symbol_number)==ORDER_TYPE_SELL && 
         close[symbol_number].value[1]<pos.last_deal_price-CorrectValueBySymbolDigits(VolumeIncreaseStep[symbol_number]*symb.point))
         return(ORDER_TYPE_SELL);

      //---    :
      if(pos.type==POSITION_TYPE_SELL && 
         GetSignal(symbol_number)==ORDER_TYPE_BUY)
         return(ORDER_TYPE_BUY);
      //---
      if(pos.type==POSITION_TYPE_BUY && 
         GetSignal(symbol_number)==ORDER_TYPE_BUY && 
         close[symbol_number].value[1]>pos.last_deal_price+CorrectValueBySymbolDigits(VolumeIncreaseStep[symbol_number]*symb.point))
         return(ORDER_TYPE_BUY);
     }
//---  
   return(WRONG_VALUE);
  }
//+------------------------------------------------------------------+
//|                                 |
//+------------------------------------------------------------------+
ENUM_ORDER_TYPE GetSignal(int symbol_number)
  {
//---    :
   if(indicator[symbol_number].value[1]<indicator[symbol_number].value[2])
      return(ORDER_TYPE_SELL);
//---    :
   if(indicator[symbol_number].value[1]>indicator[symbol_number].value[2])
      return(ORDER_TYPE_BUY);
//---  
   return(WRONG_VALUE);
  }
//+------------------------------------------------------------------+
